#!/usr/bin/env python
#
# This software is distributed with the MIT license:
# https://github.com/gkiar/clowdr/blob/master/LICENSE
#
# clowdr/controller/launcher.py
# Created by Greg Kiar on 2018-02-28.
# Email: gkiar@mcin.ca


class Endpoint:
    # TODO: document

    def __init__(self, auth):
        # TODO: document
        self.credentials = auth
